// ========================================================
// BBDrive_EyeFunc_Test1
// CBP 15-09-23
// Uses Eye Calibration function
// Eye Receptors with retinal neurons
// Uses LDR not phototransistors
// No need for inversion neurons
// ========================================================

#include <Servo.h>

// Pin defines
#define LDR_L A0
#define LDR_R A1

#define BUFLEN 10

#define ServoL_pin 13
#define ServoR_pin 12

// Neural stuff -------------------------
float tau;
float inL1, inR1;
float outL1, outR1;
float outL2, outR2;
float A = 100.0;
float B = 25.0;
float offset = 30;
float inLexp, inRexp;
float inLlog, inRlog;

float corrFacL = 1.0;//1000.0/948.0;
float corrFacR = 1.0;//1000.0/923.0;

// ODE stuff ----------------
float u[2];
float dudt[2];
float deltaT;

float bufferL[BUFLEN];
float bufferR[BUFLEN];
float avL;
float avR;

// Motor stuff --------------
Servo servoL;
Servo servoR; 
float driveL, driveR;

void calcAverage(){
  bufferL[0] = inL1;
  bufferR[0] = inR1;

  avL = 0;
  avR = 0;
  
  for(int i=0;i<BUFLEN;i++){
    avL += bufferL[i];
    avR += bufferR[i];
  }
  avL = avL / BUFLEN;
  avR = avR / BUFLEN;
  
  for(int i=BUFLEN-1;i > 0;i--){
    bufferL[i] = bufferL[i-1];
    bufferR[i] = bufferR[i-1];
  }
  
}

void setup() {
  Serial.begin(38400);
  deltaT = 0.01;
  tau = 0.01;
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
}

void loop() {

  inL1 = (float)analogRead(LDR_L)*corrFacL;
  inR1 = (float)analogRead(LDR_R)*corrFacR;   
  
  calcAverage();

  // calc exponential
  inLexp = 12.683*exp(inL1*0.0064);
  inRexp = 12.683*exp(inR1*0.0064);

  // calc log
  inLlog = 0.0064*inL1 + 2.5403;
  inRlog = 0.0064*inR1 + 2.5403;
  
  compute_retina_neurons(inLexp, inRexp);
  // Useful alt prints to check behaviour of processing steps
   cprintf("inL1 = %f,  inR1 = %f  ",inL1,inR1);
   cprintf("inLexp = %f,  inRexp = %f",inLexp,inRexp);
   //cprintf("  inLlog = %f,  inRlog = %f\n",inLlog,inRlog);
      
   //cprintf("  avL = %f, avR = %f\n",avL,avR);   
  // cprintf("u[0] = %f, u[1] = %f\n",u[0],u[1]);
   cprintf("outL1 = %f, outR1 = %f\n",outL1,outR1);

  driveMotors();
  delay(100);
}

void driveMotors(){
  // Try using outL2 and out R2;
  driveL = 50*outR1;
  driveR = 50*outL1;
  driveServos(driveL,driveR);
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}

void compute_retina_neurons(float inL, float inR){
  // Retinal layer
  dudt[0] = (-u[0] + inL - offset)/tau;
  dudt[1] = (-u[1] + inR - offset)/tau;
  u[0] += dudt[0] * deltaT;
  u[1] += dudt[1] * deltaT;
  outL1 = 1.0 / (1 + exp(-(u[0] - A)/B));
  outR1 = 1.0 / (1 + exp(-(u[1] - A)/B)); 

  // No Inversion Layer
 
}
