// ========================================================
// BBDrive_Proc_1d
// CBP 03-10-23
// Procedural Braitenberg Vehicle 2b
// Log Interpolation
// ========================================================

#include <Servo.h>

// Pin defines
#define LDR_L A0
#define LDR_R A1
#define LED_PIN 4

#define ServoL_pin 13
#define ServoR_pin 12

// Motor stuff --------------
Servo servoL;
Servo servoR;
float driveL, driveR;
float driveLow = 10;
float driveHigh = 60;

// Eye stuff ----------------
float eyeLow = 100;
float eyeHigh = 630;
float eyeL, eyeR;

// Interpolator inputs ------
float interpInL;
float interpInR;

void setup() { // --------------------------------------
  
  delay(2000);
  Serial.begin(9600);

  pinMode(LED_PIN,OUTPUT);
  digitalWrite(LED_PIN,LOW);
  
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);

} // ----------------------------------------------------

void loop() { // ----------------------------------------

  eyeL = (float)analogRead(LDR_L);
  eyeR = (float)analogRead(LDR_R);

  // make the eye-motor connexions here
  //interpInL = eyeR;
  //interpInR = eyeL;

  interpInL = eyeR;
  interpInR = eyeL;

  driveL = interpolateLog(interpInL,eyeLow,eyeHigh,driveLow,driveHigh);
  driveR = interpolateLog(interpInR,eyeLow,eyeHigh,driveLow,driveHigh);

  driveServos(driveL,driveR);

  cprintf("eyeL = %f, eyeR = %f, driveL = %f, driveR = %f\n",eyeL,eyeR,driveL,driveR);
  
} // ----------------------------------------------------

float interpolateLog(float _eyeInput,
                  float _eyeLow,
                  float _eyeHigh,
                  float _driveLow,
                  float _driveHigh){

  float driveOutput;
  float A;
  float B;
  
  // constrain inputs within allowed range
  if(_eyeInput < _eyeLow) _eyeInput = _eyeLow;
  if(_eyeInput > _eyeHigh) _eyeInput = _eyeHigh;

A = (_driveHigh - _driveLow)/(log(_eyeHigh) - log(_eyeLow));
B = (_driveLow + _driveHigh) - A*(log(_eyeLow)+log(_eyeHigh));
B = B/2;

driveOutput = A*log(_eyeInput) + B;
  

  return driveOutput;
}

void driveServos(float _driveL, float _driveR) {
  _driveR = _driveR*0.8;
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
