// ========================================================
// BBDrive_Test_2_exp
// from BBDrive_Test2
// CBP 15_09_23
// addition of exponential fn
// motorR tweak to reduce over-run
// ========================================================

#include <Servo.h>

// Pin defines
#define LDR_L A0
#define LDR_R A1

#define BUFLEN 10 // For averaging (see note above)

#define STATE_RUN 1
#define STATE_END 2

#define ServoL_pin 13
#define ServoR_pin 12

// Neural stuff -------------------------
float tau;
float inL1, inR1;
float outL1, outR1;
float outL2, outR2;
float A = 100.0;
float B = 25.0;
float inLexp, inRexp;

// Insert backgrounds here --------------
float offsetL = 30;
float offsetR = 30;
// --------------------------------------

float C = 25;
float D = 15;

float corrFacL = 1.0;//631.0/596.0;
float corrFacR = 1.0;//640.0/550.0;

// ODE stuff ----------------
float u[4];
float dudt[4];
float deltaT;

float bufferL[BUFLEN];
float bufferR[BUFLEN];
float avL;
float avR;
float inL, inR;

unsigned long startTime, duration;
int state;
float ttime;

// Motor stuff --------------
Servo servoL;
Servo servoR;
float driveL, driveR;
float maxDrive = 60;
bool firstPass;

void calcAverage() {
  bufferL[0] = inL;
  bufferR[0] = inR;

  avL = 0;
  avR = 0;

  for (int i = 0; i < BUFLEN; i++) {
    avL += bufferL[i];
    avR += bufferR[i];
  }
  avL = avL / BUFLEN;
  avR = avR / BUFLEN;

  for (int i = BUFLEN - 1; i > 0; i--) {
    bufferL[i] = bufferL[i - 1];
    bufferR[i] = bufferR[i - 1];
  }

}

void setup() {
  
  delay(5000);
  Serial.begin(38400);
  deltaT = 0.01;
  tau = 0.01;
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);

  startTime = millis();
  duration = 100 * 1000; // Set duration

  ttime = 0;
  state = STATE_RUN;

}

void loop() {

  switch (state) {
    case STATE_RUN:

      inL = (float)analogRead(LDR_L) * corrFacL;
      inR = (float)analogRead(LDR_R) * corrFacR;

      // calc exponential
      inLexp = 12.683*exp(inL*0.0064);
      inRexp = 12.683*exp(inR*0.0064);
  
      calcAverage();

      inL1 = inLexp;
      inR1 = inRexp;
      
      compute_all_neurons(inL1, inR1);

      cprintf("inL1 = %f, inR1 = %f, outL1 = %f, outR1 = %f outL2 = %f, outR2 = %f\n",inL1,inR1,outL1,outR1,outL2,outR2);

      driveMotors();
      delay(100);

      ttime = (float)(millis() - startTime) / 1000.0;
      if ( (millis() - startTime) > duration ) {
        state = STATE_END;
      }
      break;

    case STATE_END:
      break;
  }
}

void driveMotors() {
  driveL = outR2;
  driveR = outL2;
  driveServos(driveL, driveR);
}

void driveServos(float _driveL, float _driveR) {
  _driveR = _driveR*0.8;
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}

void compute_all_neurons(float inL, float inR) {
  // Retinal layer
  dudt[0] = (-u[0] + inL - offsetL) / tau;
  dudt[1] = (-u[1] + inR - offsetR) / tau;
  u[0] += dudt[0] * deltaT;
  u[1] += dudt[1] * deltaT;
  outL1 = 1.0 / (1 + exp(-(u[0] - A) / B));
  outR1 = 1.0 / (1 + exp(-(u[1] - A) / B));

  // No Inversion Layer

  // Motor neurons using shunting
  dudt[2] = (-u[2] + maxDrive * outL1) / tau;
  dudt[3] = (-u[3] + maxDrive * outR1) / tau;
  u[2] += dudt[2] * deltaT;
  u[3] += dudt[3] * deltaT;

  outL2 = maxDrive / (1 + exp(-(u[2] - C) / D));
  outR2 = maxDrive / (1 + exp(-(u[3] - C) / D));

}
