// ========================================================
// BBDrive_Test_4a_exp
// Vehicle 2b / 2a
// Voltage to luminance conversion
// CBP 15-09-23
// Eye Receptors with retinal neurons
// L and R eye measure and remember ambient light for time
// (neural solution)
// Motor neurons with shunting multiplication
// Uses LDR not phototransistors
// No need for inversion neurons
// ========================================================

#include <Servo.h>

// Pin defines
#define LDR_L A0
#define LDR_R A1
#define LDR_A A2

#define BUFLEN 10

#define ServoL_pin 13
#define ServoR_pin 12

// Neural stuff -------------------------
float tau;
float inL1, inR1;
float inMemL, inMemR;
float outL1, outR1;
float outL2, outR2;
float A = 100.0;
float B = 25.0;
//float offset = 500;
float C = 25;
float D = 15;

float corrFacL = 1000.0/948.0;
float corrFacR = 1000.0/923.0;

// ODE stuff ----------------
float u[6];
float dudt[6];
float deltaT;

float bufferL[BUFLEN];
float bufferR[BUFLEN];
float bufferA[BUFLEN];
float avL;
float avR;

float memL, memR;

// Motor stuff --------------
Servo servoL;
Servo servoR; 
float driveL, driveR;
float maxDrive = 60;

// Other stuff --------------
unsigned long startTime,   currTime;

void calcAverage(){
  bufferL[0] = inL1;
  bufferR[0] = inR1;
  
  avL = 0;
  avR = 0;
  
  for(int i=0;i<BUFLEN;i++){
    avL += bufferL[i];
    avR += bufferR[i];
  }
  avL = avL / BUFLEN;
  avR = avR / BUFLEN;
    
  for(int i=BUFLEN-1;i > 0;i--){
    bufferL[i] = bufferL[i-1];
    bufferR[i] = bufferR[i-1];  
  }
  
}

void setup() {
  pinMode(4,OUTPUT);
  Serial.begin(9600);
  deltaT = 0.01;
  tau = 0.01;
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);

  startTime = millis();

}

void loop() {

 // inL1 = (float)analogRead(LDR_L)*corrFacL;
 // inR1 = (float)analogRead(LDR_R)*corrFacR;   

  inL1 = (float)analogRead(LDR_L);
  inR1 = (float)analogRead(LDR_R);      
    
  calcAverage();
  
  compute_all_neurons(inL1, inR1);
  // Useful alt prints to check behaviour of processing steps
  // cprintf("inL1 = %f,  inR1 = %f, inA = %f\n",inL1,inR1,inA);
  // cprintf("u[0] = %f,  u[1] = %f, u[4] = %f\n",u[0],u[1],u[4]);
  cprintf("memL = %f, memR = %f, u[0] = %f,  u[1] = %f\n",memL, memR,u[0],u[1]);
  // cprintf("avL = %f, avR = %f\n",avL,avR);   
  // cprintf("u[0] = %f, u[1] = %f\n",u[0],u[1]);
  //cprintf("outL1 = %f, outR1 = %f\n",outL1,outR1);
  //cprintf("u[2] = %f, u[3] = %f\n",u[2],u[3]);
  //cprintf("outL2 = %f, outR2 = %f\n",outL2,outR2);

  driveMotors();
  delay(100);
}

void driveMotors(){
  driveL = outR2;
  driveR = outL2;
  driveServos(driveL,driveR);
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}

void compute_all_neurons(float inL, float inR){
  
  // memory neurons turn on ------------------
  currTime = millis();
  if(currTime - startTime < 10000) {
    // Send unput values to neurons 4 & 5 to
    // save the values.
    inMemL = inL;
    inMemR = inR;
    digitalWrite(4,HIGH);
    // Prevent robot from moving
    servoL.detach();
    servoR.detach();
  } else {
    servoL.attach(ServoL_pin);
    servoR.attach(ServoR_pin);
    // Stop changing the memory neurons 4 & 5
    inMemL = u[4];
    inMemR = u[5];
    digitalWrite(4,LOW);
  }

  // Additional memory neurons ---------------
  dudt[4] = (-u[4] + inMemL)/tau;
  u[4] += dudt[4]*deltaT;
  dudt[5] = (-u[5] + inMemR)/tau;
  u[5] += dudt[5]*deltaT;

  // used stored values as offsets for neurons
  // 0 & 1
  memL = u[4];
  memR = u[5];
    
  // Retinal layer ---------------------------
  dudt[0] = (-u[0] + inL - memL)/tau;
  dudt[1] = (-u[1] + inR - memR)/tau;
  u[0] += dudt[0] * deltaT;
  u[1] += dudt[1] * deltaT;
  outL1 = 1.0 / (1 + exp(-(u[0] - A)/B));
  outR1 = 1.0 / (1 + exp(-(u[1] - A)/B)); 

  // No Inversion Layer

  // Motor neurons using shunting -----------
  dudt[2] = (-u[2] + maxDrive*outL1)/tau;
  dudt[3] = (-u[3] + maxDrive*outR1)/tau;
  u[2] += dudt[2] * deltaT;
  u[3] += dudt[3] * deltaT;

  outL2 = maxDrive / (1 + exp(-(u[2] - C)/D));
  outR2 = maxDrive / (1 + exp(-(u[3] - C)/D)); 

}
