// ========================================================
//
// CBP_2403_R_ArcTest_1
// CBP 30-08-23
// Moves along arc of given radius and angle  with correction
// Status Release
//
// ========================================================

#include <stdio.h>
#include <Servo.h>

// Define pins ---------------
#define ServoL_pin 13
#define ServoR_pin 12
#define EncoderL_pin 3
#define EncoderR_pin 2
#define LED_Red_pin 4
#define LED_Yel_pin 5

// Define constants ----------
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 115.0
#define motorFactor (5.0/3.0)
#define dx (wheelRad*2*PI/64)

// Global variables ----------
Servo servoL;
Servo servoR; 
float driveL, driveR;

volatile unsigned long countL = 0;
volatile unsigned long countR = 0;

// Parameters for the arc
float desRadius = 0.0;
float desDegrees = 0.0;
float desTheta = 0.0;

unsigned long nL, nR;
float nLf, nRf;

void setup() {
  Serial.begin(9600);

  cprintf(">>> Initializing stuff ..................\n");
  //pinMode(10,INPUT);
  //pinMode(2,INPUT);
  //pinMode(11,INPUT);
  servoL.attach(ServoL_pin);
  servoR.attach(ServoR_pin);
  attachInterrupt(digitalPinToInterrupt(EncoderL_pin),ISR_Left,CHANGE);
  attachInterrupt(digitalPinToInterrupt(EncoderR_pin),ISR_Right,CHANGE);  

  // Set variables
  driveL = 40;
  driveR = 40;

  countL = 0;
  countR = 0;

  // Describe the desired arc
  desRadius = 300;
  desDegrees = 90.0;
  desTheta = desDegrees*(PI/180.0);

  // Calc the desired steps, first in floats
  nLf = (desRadius - axleLen/2.0)*desTheta/dx;
  nRf = (desRadius + axleLen/2.0)*desTheta/dx;
  nL = (unsigned long) nLf;
  nR = (unsigned long) nRf;
  
  cprintf("Desired steps = nL = %l, nR = %l\n",nL,nR);

  // Insert corrected nLf, nRf from Octave here
  nLf = 118.6;
  nRf = 179.8;
  nL = (unsigned long)nLf;
  nR = (unsigned long)nRf;

  // Calc the drives so both motors run for the same time
  driveR = driveL * nRf / nLf;
  
  cprintf("Corrected steps: nL = %l, nR = %l\n",nL,nR);
  cprintf("Corrected drives: driveL = %f, driveR = %f\n",driveL,driveR);

  delay(5000);
          
  cprintf(">>> Entering loop() .................\n");
}



void loop() {


  while( (countL < nL) && (countR < nR) ){
      driveServos(driveL,driveR);
      delay(200);
      if(DEBUG) cprintf("CountL = %l, countR = %l\n", countL,countR);
  }
  driveServos(0,0);
  cprintf("FINAL CountL = %l, countR = %l\n",countL,countR);
  cprintf("FINAL delSL = %f, delSR = %f\n",dx*countL, dx*countR);
  
  delay(10000);
}

void ISR_Left() {
  countL++;
}

void ISR_Right() {
  countR++;
}

void driveServos(float _driveL, float _driveR) {
  servoL.writeMicroseconds(1500 + (int)_driveL);
  servoR.writeMicroseconds(1500 - (int)_driveR);
}
