// ======================================================
// CBP_3402_R_Dist_Test
// 17-10-21
// Uses LUT to convert width to distance
// FBO_CBP_Calibrate_Camera.ino
// FB0 CBP 13-07021
// CBP Mod 14-07-21
// ======================================================

#include <Pixy2.h>
#include <Servo.h>

// Define pins ---------------
#define ServoL_pin 13
#define ServoR_pin 12
#define ServoTurret_pin 11

// Define constants ----------
#define DEBUG true
#define wheelDiam 66.0
#define wheelRad (wheelDiam/2.0)
#define axleLen 104.0

// Global variables ----------
Servo servoL;
Servo servoR; 
Servo servoTurret;

float driveL, driveR;

// This is the main Pixy object 
Pixy2 pixy;

// Pixy variables
uint16_t width;
uint16_t height;
uint16_t color;
uint8_t index;
uint8_t x;
uint8_t y;

int offset;

void setup() {
  Serial.begin(9600);
  cprintf(">>> Initializing stuff ..................\n");
  Serial.begin(9600);
  Serial.print("Starting...\n");

  // Make sure camera is pointing forwards.  
  // Apply your offset here
  offset = 0;
  servoTurret.attach(ServoTurret_pin);
  for(int angle = 0; angle < 180; angle++){
    servoTurret.write(angle + offset);
    delay(10);
  }
  // Now set it to 90 degs pointing forwards
  servoTurret.write(90 + offset);
  delay(1000);
  servoTurret.detach();

  // Init the camera
  pixy.init();

  cprintf(">>> Pausing 2 secs ......................\n");
  delay(2000);
  cprintf(">>> Starting  ...........................\n"); 

}

void loop() {
  float dist;
  // Following line will return just one block, the largest
  pixy.ccc.getBlocks(true, 1, 1);

  // If we have a block then print out its width
  if (pixy.ccc.numBlocks > 0) {
    width = pixy.ccc.blocks[0].m_width;
    dist = getDistanceFromObject(width);
    cprintf("width = %d (pel) dist = %f (mm)\n",width,dist);   
 }
  
}

// Change the parameters in the following to your values
float getDistanceFromObject(uint16_t _width) {
  float dist;
  dist = 15276.2/((float)_width) - 38.3;
  return dist;
}
