// ========================================================
// Severus_RTOS_Test
// CBP FBO 08-12-23
// Tests 2-tasks: movement and husky read
// I2C Protocol
// ========================================================

#include <Arduino_FreeRTOS.h>
#include "FBO_CBP_Stepper_B.h"
#include "HUSKYLENS.h"
#include <Wire.h>

#define STATE_FORWARD 1
#define STATE_PIVOT 2

int state;

HUSKYLENS husky;

int id;

FBO_CBP_Stepper_B Severus(200,1);

AF_Stepper motor1(200,1);
AF_Stepper motor2(200,2);

float dx = 2 * PI * 33 / 200;
float axleLength = 140;
float vMin = 20;
float vMax = 50;

TaskHandle_t handle1;
TaskHandle_t handle2;

void setup(){

  //mySerial.begin(9600);
  Serial.begin(250000);

  Severus.setRobotParams(dx,axleLength);
  
  Wire.begin();

  while( !husky.begin(Wire)){
    Serial.println("Check General Seetings >> Protocol Type >> I2C");
    delay(100);
  }
  
  Serial.println("Husky Configured");
  husky.writeAlgorithm(ALGORITHM_TAG_RECOGNITION);

   if (!husky.request()) Serial.println(F("Fail to request data from HUSKYLENS, recheck the connection!"));
    else if(!husky.isLearned()) Serial.println(F("Nothing learned, press learn button on HUSKYLENS to learn one!"));
    else if(!husky.available()) Serial.println(F("No block or arrow appears on the screen!"));
  
  HUSKYLENSResult result = husky.read();
  Serial.println(String()+F("ID = ") + result.ID);

  state = STATE_FORWARD;

  // Create RTOS tasks
  xTaskCreate(CheckHusky, "Task1", 200, NULL, 1, &handle1);
  xTaskCreate(Movement, "Task2", 200, NULL, 1, &handle2);

}

void CheckHusky(void *pvParameters) { // --------- task ----
  while(1){
    cprintf("Husky check\n");
    husky.request();
    HUSKYLENSResult result = husky.read();
    cprintf(">>>>>>> CheckHusky <<<<< id = %d\n",result.ID);
    if(result.ID == 2) {
      Severus.bSoftInterrupt = true;
      state = STATE_PIVOT;
    }  
    vTaskDelay(1000/portTICK_PERIOD_MS);   
  }

}

void Movement(void *pvParameters){  // ------- task ------
(void) pvParameters;
  while(1){
    cprintf("Movement state = %d\n",state);
    switch(state) {
      case STATE_FORWARD:
        Severus.straightLine( 2000,vMin,vMax,true,true,motor1,motor2);
      break;

      case STATE_PIVOT:
        Severus.pivot( 90,vMin,vMax,true,true,motor1,motor2);
        state = STATE_FORWARD;
      break;
    } 
    vTaskDelay(1000/portTICK_PERIOD_MS);    
  }

}

void loop() {
                        
}
