#include "FBO_CBP_Stepper_B.h"
#include "AFMotor.h"
#include <Arduino.h>

FBO_CBP_Stepper_B::FBO_CBP_Stepper_B(uint16_t steps, uint8_t num) {
	Stepper = new AF_Stepper(steps, num);
}



void FBO_CBP_Stepper_B::straightLine(int dist, int vMin, int vMax, 
                                     bool upRamp, bool downRamp,
									 AF_Stepper motorLeft, AF_Stepper motorRight) {
	
  unsigned long i=0;
  unsigned long steps;
  unsigned long delayVal;
  
  int dir = 1;
  
  if(dist > 0) {
	dir = FORWARD;
  } else {
	dir = BACKWARD;
  }
  
  steps = abs(dist)/dx;

  while(i <= steps) {
	delayVal = ramp(vMin, vMax, steps, 0.25, i, upRamp, downRamp);
	int m1 = motorLeft.onestep(dir, SINGLE);
    int m2 = motorRight.onestep(dir, SINGLE);
    delayMicroseconds(delayVal);
    i++;
  } 
}

unsigned long FBO_CBP_Stepper_B::ramp(int vMin, int vMax, unsigned long N, float alpha, unsigned long n, bool upRamp, bool downRamp) {

    float vely;
    unsigned long uSDelay;

    //cprintf("vMin = %d, vMax = %d, N = %l, alpha = %f, n = %d\n",vMin,vMax,N,alpha,n);

    if ((n < alpha * N) && (upRamp == true)) {
        vely = n * (vMax - vMin) / (alpha * N) + vMin;
        //cprintf("vely = %f\n",vely);
    }
    else if ((n < alpha * N) && (upRamp == false)) {
        vely = vMax;
    }

    if ((n >= alpha * N) && (n < (N - alpha * N))) {
        vely = vMax;
    }

    if ((n >= (N - alpha * N)) && (downRamp == true)) {
        vely = (N - n) * (vMax - vMin) / (alpha * N) + vMin;
    }
    else if ((n >= (N - alpha * N)) && (downRamp == false)) {
        vely = vMax;
    }

    if (upRamp == false && downRamp == false) {
        vely = vMax;
    }

    uSDelay = (unsigned long)(60 * 5000.0 / vely);

    return uSDelay;

}

void FBO_CBP_Stepper_B::pivot(float degs, int vMin, int vMax, bool upRamp, bool downRamp, AF_Stepper motorLeft, AF_Stepper motorRight) {

    float theta = (PI * degs) / 180.0;

    int dirL;
    int dirR;

    unsigned long nL, nR;
    unsigned long vL, vR;
    float sC;

    unsigned long delayVal;

    if (theta <= 0) {
        dirL = FORWARD;
        dirR = BACKWARD;
    }
    else {
        dirL = BACKWARD;
        dirR = FORWARD;
    }

    sC = axleLength / 2 * abs(theta);
    nL = sC / dx;
    nR = nL;

    int i = 0;
    while (i <= nL) {
        delayVal = ramp(vMin, vMax, nL, 0.25, i, upRamp, downRamp);

        int m1 = motorLeft.onestep(dirL, SINGLE);
        int m2 = motorRight.onestep(dirR, SINGLE);
        delayMicroseconds(delayVal);

        i++;
    }
}

void FBO_CBP_Stepper_B::arc(float R, float degs, int vMin, int vMax, bool upRamp, bool downRamp, AF_Stepper motorLeft, AF_Stepper motorRight) {
  
  float nRF, nLF;
  unsigned long nR,nL;
  float beta;
  float prepR;
  float prepL;
  unsigned long doneL,doneR;
  float theta;
  float a;
  float delayVal;

  unsigned long delayValL;
  unsigned long delayValR;
  
  a = axleLength/2.0;
  theta = degs*PI/180.0;

  // ---------------------------------- positive theta -------------------------
  if(theta > 0.0) {
  nRF = (theta/dx) * (R - a);
  nLF = (theta/dx) * (R + a);

  nR = ceil(nRF);
  nL = ceil(nLF);

  beta = nRF/nLF;
  prepR = 0;
  doneL = 0;
  doneR = 0;

  while (doneL < nL) { 

    delayVal = ramp(vMin, vMax, nL, 0.25, doneL+1, upRamp, downRamp);      

    int m1 = motorLeft.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);
    //stepMotors(1, 0);
    doneL += 1;
    prepR += beta;
    
    if(prepR > 1.0) {
      int m2 = motorRight.onestep(FORWARD, SINGLE);   
      delayMicroseconds(delayVal);
      //stepMotors(0,1);
      doneR += 1;
      prepR -= 1.0;  
    }
  }

  if (prepR > 0.5) {
    //stepMotors(0,1);
    int m2 = motorRight.onestep(FORWARD, SINGLE);   
    delayMicroseconds(delayVal);
    doneR += 1.0;
    prepR -= 1.0;
  }
 
    
  } else { 
    
  // ---------------------------------- negative theta -------------------------
  theta = -theta;
  nLF = (theta/dx) * (R - a);
  nRF = (theta/dx) * (R + a);

  nR = ceil(nRF);
  nL = ceil(nLF);

  beta = nLF/nRF;
  prepL = 0;
  doneL = 0;
  doneR = 0;

  while (doneR < nR) {

    delayVal = ramp(vMin, vMax, nR, 0.25, doneR+1, upRamp, downRamp);   
    
    int m2 = motorRight.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);

    doneR += 1;
    prepL += beta;
    
    if(prepL > 1.0) {
      int m1 = motorLeft.onestep(FORWARD, SINGLE); 
      delayMicroseconds(delayVal);
      doneL += 1;
      prepL -= 1.0;  
    }
  }

  if (prepL > 0.5) {
    int m1 = motorLeft.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);
    doneL += 1.0;
    prepL -= 1.0;
  }
  
  }
}

// Move depending on error
void FBO_CBP_Stepper_B::moveABitNew(float error, int vFwd, AF_Stepper motorLeft, AF_Stepper motorRight) {

	unsigned int nL, nR;
	unsigned long delayVal;
	unsigned long stretchDelay;

	delayVal = (unsigned long) (60 * 5000.0 / vFwd);
	stretchDelay = (unsigned long)(slowdown * delayVal);
	
	//Serial.print("error = ");Serial.println(error);
	
	if(error > deadBand) {
		nL = 1;
		nR = ceil(gain*error);
		//Serial.print("nL = ");Serial.print(nL);Serial.print(" nR = ");Serial.println(nR);
		// Need outer loop to get left mtor to turn
		for(int j=0; j < 4; j++){
			int m1 = motorLeft.onestep(FORWARD,SINGLE);
			delayMicroseconds(stretchDelay);
			for(int i=0;i<nR;i++){
				int  m2 = motorRight.onestep(FORWARD,SINGLE);
				delayMicroseconds(stretchDelay);
			}		
		}	
	} else if(error < -deadBand){
		nR = 1;
		nL = ceil(-gain*error);
		//Serial.print("nL = ");Serial.print(nL);Serial.print(" nR = ");Serial.println(nR);
		for(int j=0; j < 4; j++){
			int m2 = motorRight.onestep(FORWARD,SINGLE);
			delayMicroseconds(stretchDelay);
			for(int i=0;i<nL;i++){
				int m1 = motorLeft.onestep(FORWARD,SINGLE);
				delayMicroseconds(stretchDelay);
			}		
		}
	} else {
		for (int i = 0; i < 4; i++)  {
			int m2 = motorLeft.onestep(FORWARD, SINGLE); 
			delayMicroseconds(delayVal);
			int m1 = motorRight.onestep(FORWARD, SINGLE); 
        	delayMicroseconds(delayVal);
		}
	}

}


// Move depending on error
void FBO_CBP_Stepper_B::moveABit(float error, float deltaT, int vMin, int vMax, AF_Stepper motorLeft, AF_Stepper motorRight) {
  float vLF, vRF;
  float nLF, nRF;
  float beta;
  float prepR, prepL;
  int speedL, speedR;
  int nR,nL;
  int  vL,vR;
  int doneL,doneR;
  unsigned long delayVal;
  int L = 180; //mm from axlle to camera
  float theta;
  float R = 200;  // radius of correction arc
  R = 250;
  R = radMoveABit;
  L = distCameraAxle;
  //float axleLength = 140;
  bool upRamp = false;
  bool downRamp = false;
  
  float a = axleLength/2.0;
  theta = -asin(error*22.0/L);

  float vReduce = 0.5;
  
  //float deadBand = 0.1;
  
if(error < -deadBand) {
	

  nRF = (theta/dx) * (R - a);
  nLF = (theta/dx) * (R + a);

  nR = ceil(nRF);
  nL = ceil(nLF);
  
  //Serial.print("theta = ");Serial.println(theta);
  //Serial.print("nLF = ");Serial.println(nLF);
  //Serial.print("nRF = ");Serial.println(nRF);
  
  beta = nRF/nLF;
  prepR = 0;
  doneL = 0;
  doneR = 0;

  while (doneL < nL) { 

    delayVal = ramp(vMin, vReduce*vMax, nL, 0.25, doneL+1, upRamp, downRamp);      

    int m1 = motorLeft.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);
    doneL += 1;
    prepR += beta;
    
    if(prepR > 1.0) {
      int m2 = motorRight.onestep(FORWARD, SINGLE);   
      delayMicroseconds(delayVal);
      doneR += 1;
      prepR -= 1.0;  
    }
  }

  if (prepR > 0.5) {
    int m2 = motorRight.onestep(FORWARD, SINGLE);   
    delayMicroseconds(delayVal);
    doneR += 1.0;
    prepR -= 1.0;
  }
 
  
}
else if(error > deadBand) {
  // ---------------------------------- negative theta -------------------------
  theta = -theta;
  nLF = (theta/dx) * (R - a);
  nRF = (theta/dx) * (R + a);

  nR = ceil(nRF);
  nL = ceil(nLF);

  //Serial.print("theta = ");Serial.println(theta);
  //Serial.print("nLF = ");Serial.println(nLF);
  //Serial.print("nRF = ");Serial.println(nRF);
  
  beta = nLF/nRF;
  prepL = 0;
  doneL = 0;
  doneR = 0;

  while (doneR < nR) {

    delayVal = ramp(vMin, vReduce*vMax, nR, 0.25, doneR+1, upRamp, downRamp);   
    
    int m2 = motorRight.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);

    doneR += 1;
    prepL += beta;
    
    if(prepL > 1.0) {
      int m1 = motorLeft.onestep(FORWARD, SINGLE); 
      delayMicroseconds(delayVal);
      doneL += 1;
      prepL -= 1.0;  
    }
  }

  if (prepL > 0.5) {
    int m1 = motorLeft.onestep(FORWARD, SINGLE); 
    delayMicroseconds(delayVal);
    doneL += 1.0;
    prepL -= 1.0;
  }
  
  }
  
  
else {


delayVal = (unsigned long) (60 * 5000.0 / vMax);

  for (int i = 0; i < 5; i++)
  {
	int m2 = motorLeft.onestep(FORWARD, SINGLE); 
  	        delayMicroseconds(delayVal);
	int m1 = motorRight.onestep(FORWARD, SINGLE); 
        	delayMicroseconds(delayVal);

}
 
}
}

    void FBO_CBP_Stepper_B::setDistCameraAxle(float _dist){
		distCameraAxle = _dist;
	}

	void FBO_CBP_Stepper_B::setRadMoveABit(float _rad) {
		radMoveABit = _rad;
	}
	
	void FBO_CBP_Stepper_B::setDeadBand(float _deadBand) {
		deadBand = _deadBand;
	}
	
	void FBO_CBP_Stepper_B::setGain(float _gain) {
		gain = _gain;
	}
	
	void FBO_CBP_Stepper_B::setSlowDown(float _slowdown) {
		slowdown = _slowdown;
	}
	
	void FBO_CBP_Stepper_B::setRobotParams(float _dx, float _axleLength) {
		dx = _dx;
		axleLength = _axleLength;
	}
	
	